# -*- coding: utf-8 -*-

# Copyright 2009, 2010 Institute for the Study of the Ancient World, New York
# University

"""
The assumption is that this transliterator should work according to the rules
of the Barrington Atlas. In the introduction to the Map-by-Map Directory (p.
vii) its Greek transliteration rules are described as follows:

Ancient Greek names are transcribed literally, though without marking accents
or long vowels. Th is used for theta, k for kappa, x for xi, ch for chi, ps for
psi. Combinations with gamma such as gg or gk become ng and nk, etc. Upsilon
with another vowel remains u (thus au for alpha + upsilon); otherwise it
normally becomes y. Rho’s rough breathing is optional for initial rho (which
can thus appear as either Rh or R), but elsewhere in the word it is omitted in
the case of names attested only in Greek. Subscripts are ignored.

The current code does some of this, but doesn't live up to all aspects,
especially dealing with two-letter combinations; needs a major rework including
some enforced assumptions about unicode normalization form, capitalization and
so forth
"""

grek_unaccented_capital = {
    u"\u0391" : "A",    # Greek capital letter alpha
    u"\u0392" : "B",    # Greek capital letter beta
    u"\u0393" : "G",    # Greek capital letter gamma
    u"\u0394" : "D",    # Greek capital letter delta
    u"\u0395" : "E",    # Greek capital letter epsilon
    u"\u0396" : "Z",    # Greek capital letter zeta
    u"\u0397" : "E",    # Greek capital letter eta
    u"\u0398" : "TH",   # Greek capital letter theta
    u"\u0399" : "I",    # Greek capital letter iota
    u"\u039A" : "K",    # Greek capital letter kappa
    u"\u039B" : "L",    # Greek capital letter lamda
    u"\u039C" : "M",    # Greek capital letter mu
    u"\u039D" : "N",    # Greek capital letter nu
    u"\u039E" : "X",   # Greek capital letter xi
    u"\u039F" : "O",    # Greek capital letter omicron
    u"\u03A0" : "P",    # Greek capital letter pi
    u"\u03A1" : "R",    # Greek capital letter rho
    u"\u03A3" : "S",    # Greek capital letter sigma
    u"\u03A4" : "T",    # Greek capital letter tau
    u"\u03A5" : "U",    # Greek capital letter upsilon
    u"\u03A6" : "PH",   # Greek capital letter phi
    u"\u03A7" : "CH",   # Greek capital letter chi
    u"\u03A8" : "PS",   # Greek capital letter psi
    u"\u03A9" : "O",    # Greek capital letter omega
    u"\u03F9" : "S"     # Greek capital lunate sigma symbol
}

grek_unaccented_small = {
    u"\u03B1" : "a",    # Greek small letter alpha
    u"\u03B2" : "b",    # Greek small letter beta
    u"\u03B3" : "g",    # Greek small letter gamma
    u"\u03B4" : "d",    # Greek small letter delta
    u"\u03B5" : "e",    # Greek small letter epsilon
    u"\u03B6" : "z",    # Greek small letter zeta
    u"\u03B7" : "e",    # Greek small letter eta
    u"\u03B8" : "th",   # Greek small letter theta
    u"\u03B9" : "i",    # Greek small letter iota
    u"\u03BA" : "k",    # Greek small letter kappa
    u"\u03BB" : "l",    # Greek small letter lamda
    u"\u03BC" : "m",    # Greek small letter mu
    u"\u03BD" : "n",    # Greek small letter nu
    u"\u03BE" : "x",   # Greek small letter xi
    u"\u03BF" : "o",    # Greek small letter omicron
    u"\u03C0" : "p",    # Greek small letter pi
    u"\u03C1" : "r",    # Greek small letter rho
    u"\u03C2" : "s",    # Greek small letter final sigma
    u"\u03C3" : "s",    # Greek small letter sigma
    u"\u03C4" : "t",    # Greek small letter tau
    u"\u03C5" : "u",    # Greek small letter upsilon
    u"\u03C6" : "ph",   # Greek small letter phi
    u"\u03C7" : "ch",   # Greek small letter chi
    u"\u03C8" : "ps",   # Greek small letter psi
    u"\u03C9" : "o"     # Greek small letter omega
}

grek_accented_modern_capital = {
    u"\u0386" : "A",    # Greek capital letter alpha with tonos
    u"\u0388" : "E",    # Greek capital letter epsilon with tonos
    u"\u0389" : "E",    # Greek capital letter eta with tonos
    u"\u038A" : "I",    # Greek capital letter iota with tonos
    u"\u038C" : "O",    # Greek capital letter omicron with tonos
    u"\u038E" : "U",    # Greek capital letter upsilon with tonos
    u"\u038F" : "O",    # Greek capital letter omega with tonos
    u"\u03AA" : "I",    # Greek capital letter iota with dialytika
    u"\u03AB" : "U"     # Greek capital letter upsilon with dialytika
}

grek_accented_modern_small = {
    u"\u0390" : "i",    # Greek small letter iota with dialytika and tonos
    u"\u03AC" : "a",    # Greek small letter alpha with tonos
    u"\u03AD" : "e",    # Greek small letter epsilon with tonos
    u"\u03AE" : "e",    # Greek small letter eta with tonos
    u"\u03AF" : "i",    # Greek small letter iota with tonos
    u"\u03B0" : "u",    # Greek small letter upsilon with tonos
    u"\u03CA" : "i",    # Greek small letter iota with dialytika
    u"\u03CB" : "u",    # Greek small letter upsilon with dialytika
    u"\u03CC" : "o",    # Greek small letter omicron with tonos
    u"\u03CD" : "u",    # Greek small letter upsilon with tonos
    u"\u03CE" : "o"     # Greek small letter omega with tonos
}

grek_accented_ancient_capital = {
    u"\u1F08" : "A",   # Greek capital letter alpha with psili (smooth breathing)
    u"\u1F09" : "A",   # Greek capital letter alpha with dasia (rough breathing)
    u"\u1F0A" : "A",   # Greek capital letter alpha with psili and varia (smooth breathing and grave accent)
    u"\u1F0B" : "A",   # Greek capital letter alpha with dasia and varia (rough breathing and grave accent)
    u"\u1F0C" : "A",   # Greek capital letter alpha with psili and oxia (smooth breathing and acute accent)
    u"\u1F0D" : "A",   # Greek capital letter alpha with dasia and oxia (rough breathing and acute accent)
    u"\u1F0E" : "A",   # Greek capital letter alpha with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F0F" : "A",   # Greek capital letter alpha with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F18" : "E",   # Greek capital letter epsilon with psili (smooth breathing)
    u"\u1F19" : "E",   # Greek capital letter epsilon with dasia (rough breathing)
    u"\u1F1A" : "E",   # Greek capital letter epsilon with psili and varia (smooth breathing and grave accent)
    u"\u1F1B" : "E",   # Greek capital letter epsilon with dasia and varia (rough breathing and grave accent)
    u"\u1F1C" : "E",   # Greek capital letter epsilon with psili and oxia (smooth breathing and acute accent)
    u"\u1F1D" : "E",   # Greek capital letter epsilon with dasia and oxia (rough breathing and acute accent)
    u"\u1F28" : "E",   # Greek capital letter eta with psili (smooth breathing)
    u"\u1F29" : "E",   # Greek capital letter eta with dasia (rough breathing)
    u"\u1F2A" : "E",   # Greek capital letter eta with psili and varia (smooth breathing and grave accent)
    u"\u1F2B" : "E",   # Greek capital letter eta with dasia and varia (rough breathing and grave accent)
    u"\u1F2C" : "E",   # Greek capital letter eta with psili and oxia (smooth breathing and acute accent)
    u"\u1F2D" : "E",   # Greek capital letter eta with dasia and oxia (rough breathing and acute accent)
    u"\u1F2E" : "E",   # Greek capital letter eta with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F2F" : "E",   # Greek capital letter eta with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F38" : "I",   # Greek capital letter iota with psili (smooth breathing)
    u"\u1F39" : "I",   # Greek capital letter iota with dasia (rough breathing)
    u"\u1F3A" : "I",   # Greek capital letter iota with psili and varia (smooth breathing and grave accent)
    u"\u1F3B" : "I",   # Greek capital letter iota with dasia and varia (rough breathing and grave accent)
    u"\u1F3C" : "I",   # Greek capital letter iota with psili and oxia (smooth breathing and acute accent)
    u"\u1F3D" : "I",   # Greek capital letter iota with dasia and oxia (rough breathing and acute accent)
    u"\u1F3E" : "I",   # Greek capital letter iota with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F3F" : "I",   # Greek capital letter iota with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F48" : "O",   # Greek capital letter omicron with psili (smooth breathing)
    u"\u1F49" : "O",   # Greek capital letter omicron with dasia (rough breathing)
    u"\u1F4A" : "O",   # Greek capital letter omicron with psili and varia (smooth breathing and grave accent)
    u"\u1F4B" : "O",   # Greek capital letter omicron with dasia and varia (rough breathing and grave accent)
    u"\u1F4C" : "O",   # Greek capital letter omicron with psili and oxia (smooth breathing and acute accent)
    u"\u1F4D" : "O",   # Greek capital letter omicron with dasia and oxia (rough breathing and acute accent)
    u"\u1F59" : "U",   # Greek capital letter upsilon with dasia (rough breathing)
    u"\u1F5B" : "U",   # Greek capital letter upsilon with dasia and varia (rough breathing and grave accent)
    u"\u1F5D" : "U",   # Greek capital letter upsilon with dasia and oxia (rough breathing and acute accent)
    u"\u1F5F" : "U",   # Greek capital letter upsilon with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F68" : "O",   # Greek capital letter omega with psili (smooth breathing)
    u"\u1F69" : "O",   # Greek capital letter omega with dasia (rough breathing)
    u"\u1F6A" : "O",   # Greek capital letter omega with psili and varia (smooth breathing and grave accent)
    u"\u1F6B" : "O",   # Greek capital letter omega with dasia and varia (rough breathing and grave accent)
    u"\u1F6C" : "O",   # Greek capital letter omega with psili and oxia (smooth breathing and acute accent)
    u"\u1F6D" : "O",   # Greek capital letter omega with dasia and oxia (rough breathing and acute accent)
    u"\u1F6E" : "O",   # Greek capital letter omega with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F6F" : "O",   # Greek capital letter omega with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F88" : "A",   # Greek capital letter alpha with psili and prosgegrammeni (smooth breathing and iota-subscript)
    u"\u1F89" : "A",   # Greek capital letter alpha with dasia and prosgegrammeni (rough breathing and iota-subscript)
    u"\u1F8A" : "A",   # Greek capital letter alpha with psili and varia and prosgegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1F8B" : "A",   # Greek capital letter alpha with dasia and varia and prosgegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1F8C" : "A",   # Greek capital letter alpha with psili and oxia and prosgegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1F8D" : "A",   # Greek capital letter alpha with dasia and oxia and prosgegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1F8E" : "A",   # Greek capital letter alpha with psili and perispomeni and prosgegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1F8F" : "A",   # Greek capital letter alpha with dasia and perispomeni and prosgegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1F98" : "E",   # Greek capital letter eta with psili and prosgegrammeni (smooth breathing and iota-subscript)
    u"\u1F99" : "E",   # Greek capital letter eta with dasia and prosgegrammeni (rough breathing and iota-subscript)
    u"\u1F9A" : "E",   # Greek capital letter eta with psili and varia and prosgegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1F9B" : "E",   # Greek capital letter eta with dasia and varia and prosgegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1F9C" : "E",   # Greek capital letter eta with psili and oxia and prosgegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1F9D" : "E",   # Greek capital letter eta with dasia and oxia and prosgegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1F9E" : "E",   # Greek capital letter eta with psili and perispomeni and prosgegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1F9F" : "E",   # Greek capital letter eta with dasia and perispomeni and prosgegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1FA8" : "O",   # Greek capital letter omega with psili and prosgegrammeni (smooth breathing and iota-subscript)
    u"\u1FA9" : "O",   # Greek capital letter omega with dasia and prosgegrammeni (rough breathing and iota-subscript)
    u"\u1FAA" : "O",   # Greek capital letter omega with psili and varia and prosgegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1FAB" : "O",   # Greek capital letter omega with dasia and varia and prosgegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1FAC" : "O",   # Greek capital letter omega with psili and oxia and prosgegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1FAD" : "O",   # Greek capital letter omega with dasia and oxia and prosgegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1FAE" : "O",   # Greek capital letter omega with psili and perispomeni and prosgegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1FAF" : "O",   # Greek capital letter omega with dasia and perispomeni and prosgegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1FB8" : "a",  # Greek capital letter alpha with vrachy
    u"\u1FB9" : "a",  # Greek capital letter alpha with macron
    u"\u1FBA" : "a",  # Greek capital letter alpha with varia
    u"\u1FBB" : "a",  # Greek capital letter alpha with oxia
    u"\u1FBC" : "a",  # Greek capital letter alpha with prosgegrammeni
    u"\u1FC8" : "E",  # Greek capital letter epsilon with varia
    u"\u1FC9" : "E",  # Greek capital letter epsilon with oxia
    u"\u1FCA" : "E",  # Greek capital letter eta with varia
    u"\u1FCB" : "E",  # Greek capital letter eta with oxia
    u"\u1FCC" : "E"  # Greek capital letter eta with prosgegrammeni
}

grek_accented_ancient_small = {
    u"\u1F00" : "a",   # Greek small letter alpha with psili (smooth breathing)
    u"\u1F01" : "a",   # Greek small letter alpha with dasia (rough breathing)
    u"\u1F02" : "a",   # Greek small letter alpha with psili and varia (smooth breathing and grave accent)
    u"\u1F03" : "a",   # Greek small letter alpha with dasia and varia (rough breathing and grave accent)
    u"\u1F04" : "a",   # Greek small letter alpha with psili and oxia (smooth breathing and acute accent)
    u"\u1F05" : "a",   # Greek small letter alpha with dasia and oxia (rough breathing and acute accent)
    u"\u1F06" : "a",   # Greek small letter alpha with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F07" : "a",   # Greek small letter alpha with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F10" : "e",   # Greek small letter epsilon with psili (smooth breathing)
    u"\u1F11" : "e",   # Greek small letter epsilon with dasia (rough breathing)
    u"\u1F12" : "e",   # Greek small letter epsilon with psili and varia (smooth breathing and grave accent)
    u"\u1F13" : "e",   # Greek small letter epsilon with dasia and varia (rough breathing and grave accent)
    u"\u1F14" : "e",   # Greek small letter epsilon with psili and oxia (smooth breathing and acute accent)
    u"\u1F15" : "e",   # Greek small letter epsilon with dasia and oxia (rough breathing and acute accent)
    u"\u1F20" : "e",   # Greek small letter eta with psili (smooth breathing)
    u"\u1F21" : "e",   # Greek small letter eta with dasia (rough breathing)
    u"\u1F22" : "e",   # Greek small letter eta with psili and varia (smooth breathing and grave accent)
    u"\u1F23" : "e",   # Greek small letter eta with dasia and varia (rough breathing and grave accent)
    u"\u1F24" : "e",   # Greek small letter eta with psili and oxia (smooth breathing and acute accent)
    u"\u1F25" : "e",   # Greek small letter eta with dasia and oxia (rough breathing and acute accent)
    u"\u1F26" : "e",   # Greek small letter eta with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F27" : "e",   # Greek small letter eta with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F30" : "i",   # Greek small letter iota with psili (smooth breathing)
    u"\u1F31" : "i",   # Greek small letter iota with dasia (rough breathing)
    u"\u1F32" : "i",   # Greek small letter iota with psili and varia (smooth breathing and grave accent)
    u"\u1F33" : "i",   # Greek small letter iota with dasia and varia (rough breathing and grave accent)
    u"\u1F34" : "i",   # Greek small letter iota with psili and oxia (smooth breathing and acute accent)
    u"\u1F35" : "i",   # Greek small letter iota with dasia and oxia (rough breathing and acute accent)
    u"\u1F36" : "i",   # Greek small letter iota with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F37" : "i",   # Greek small letter iota with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F40" : "o",   # Greek small letter omicron with psili (smooth breathing)
    u"\u1F41" : "o",   # Greek small letter omicron with dasia (rough breathing)
    u"\u1F42" : "o",   # Greek small letter omicron with psili and varia (smooth breathing and grave accent)
    u"\u1F43" : "o",   # Greek small letter omicron with dasia and varia (rough breathing and grave accent)
    u"\u1F44" : "o",   # Greek small letter omicron with psili and oxia (smooth breathing and acute accent)
    u"\u1F45" : "o",   # Greek small letter omicron with dasia and oxia (rough breathing and acute accent)
    u"\u1F50" : "u",   # Greek small letter upsilon with psili (smooth breathing)
    u"\u1F51" : "u",   # Greek small letter upsilon with dasia (rough breathing)
    u"\u1F52" : "u",   # Greek small letter upsilon with psili and varia (smooth breathing and grave accent)
    u"\u1F53" : "u",   # Greek small letter upsilon with dasia and varia (rough breathing and grave accent)
    u"\u1F54" : "u",   # Greek small letter upsilon with psili and oxia (smooth breathing and acute accent)
    u"\u1F55" : "u",   # Greek small letter upsilon with dasia and oxia (rough breathing and acute accent)
    u"\u1F56" : "u",   # Greek small letter upsilon with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F57" : "u",   # Greek small letter upsilon with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F60" : "o",   # Greek small letter omega with psili (smooth breathing)
    u"\u1F61" : "o",   # Greek small letter omega with dasia (rough breathing)
    u"\u1F62" : "o",   # Greek small letter omega with psili and varia (smooth breathing and grave accent)
    u"\u1F63" : "o",   # Greek small letter omega with dasia and varia (rough breathing and grave accent)
    u"\u1F64" : "o",   # Greek small letter omega with psili and oxia (smooth breathing and acute accent)
    u"\u1F65" : "o",   # Greek small letter omega with dasia and oxia (rough breathing and acute accent)
    u"\u1F66" : "o",   # Greek small letter omega with psili and perispomeni (smooth breathing and circumflex accent)
    u"\u1F67" : "o",   # Greek small letter omega with dasia and perispomeni (rough breathing and circumflex accent)
    u"\u1F70" : "a",   # Greek small letter alpha with oxia
    u"\u1F71" : "a",   # Greek small letter alpha with varia
    u"\u1F72" : "e",   # Greek small letter epsilon with oxia
    u"\u1F73" : "e",   # Greek small letter epsilon with varia
    u"\u1F74" : "e",   # Greek small letter eta with oxia
    u"\u1F75" : "e",   # Greek small letter eta with varia
    u"\u1F76" : "i",   # Greek small letter iota with oxia
    u"\u1F77" : "i",   # Greek small letter iota with varia
    u"\u1F78" : "o",   # Greek small letter omicron with oxia
    u"\u1F79" : "o",   # Greek small letter omicron with varia
    u"\u1F7A" : "u",   # Greek small letter upsilon with oxia
    u"\u1F7B" : "u",   # Greek small letter upsilon with varia
    u"\u1F7C" : "o",   # Greek small letter omega with oxia
    u"\u1F7D" : "o",   # Greek small letter omega with varia
    u"\u1F80" : "a",   # Greek small letter alpha with psili and ypogegrammeni (smooth breathing and iota-subscript)
    u"\u1F81" : "a",   # Greek small letter alpha with dasia and ypogegrammeni (rough breathing and iota-subscript)
    u"\u1F82" : "a",   # Greek small letter alpha with psili and varia and ypogegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1F83" : "a",   # Greek small letter alpha with dasia and varia and ypogegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1F84" : "a",   # Greek small letter alpha with psili and oxia and ypogegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1F85" : "a",   # Greek small letter alpha with dasia and oxia and ypogegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1F86" : "a",   # Greek small letter alpha with psili and perispomeni and ypogegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1F87" : "a",   # Greek small letter alpha with dasia and perispomeni and ypogegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1F90" : "e",   # Greek small letter eta with psili and ypogegrammeni (smooth breathing and iota-subscript)
    u"\u1F91" : "e",   # Greek small letter eta with dasia and ypogegrammeni (rough breathing and iota-subscript)
    u"\u1F92" : "e",   # Greek small letter eta with psili and varia and ypogegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1F93" : "e",   # Greek small letter eta with dasia and varia and ypogegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1F94" : "e",   # Greek small letter eta with psili and oxia and ypogegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1F95" : "e",   # Greek small letter eta with dasia and oxia and ypogegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1F96" : "e",   # Greek small letter eta with psili and perispomeni and ypogegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1F97" : "e",   # Greek small letter eta with dasia and perispomeni and ypogegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1FA0" : "o",   # Greek small letter omega with psili and ypogegrammeni (smooth breathing and iota-subscript)
    u"\u1FA1" : "o",   # Greek small letter omega with dasia and ypogegrammeni (rough breathing and iota-subscript)
    u"\u1FA2" : "o",   # Greek small letter omega with psili and varia and ypogegrammeni (smooth breathing and grave accent and iota-subscript)
    u"\u1FA3" : "o",   # Greek small letter omega with dasia and varia and ypogegrammeni (rough breathing and grave accent and iota-subscript)
    u"\u1FA4" : "o",   # Greek small letter omega with psili and oxia and ypogegrammeni (smooth breathing and acute accent and iota-subscript)
    u"\u1FA5" : "o",   # Greek small letter omega with dasia and oxia and ypogegrammeni (rough breathing and acute accent and iota-subscript)
    u"\u1FA6" : "o",   # Greek small letter omega with psili and perispomeni and ypogegrammeni (smooth breathing and circumflex accent and iota-subscript)
    u"\u1FA7" : "o",   # Greek small letter omega with dasia and perispomeni and ypogegrammeni (rough breathing and circumflex accent and iota-subscript)
    u"\u1FB2" : "a",   # Greek small letter alpha with varia and ypogegrammeni
    u"\u1FB3" : "a",   # Greek small letter alpha with ypogegrammeni
    u"\u1FB4" : "a",   # Greek small letter alpha with oxia and ypogegrammeni
    u"\u1FB6" : "a",  # Greek small letter alpha with perispomeni
    u"\u1FB7" : "a",  # Greek small letter alpha with perispomeni and ypogegrammeni
    u"\u1FC2" : "e",  # Greek small letter eta with varia and ypogegrammeni
    u"\u1FC3" : "e",  # Greek small letter eta with ypogegrammeni
    u"\u1FC4" : "e",  # Greek small letter eta with oxi and ypogegrammeni
    u"\u1FC6" : "e",  # Greek small letter eta with perispomeni
    u"\u1FC7" : "e",  # Greek small letter eta with perispomeni and ypogegrammeni
    u"\u1FE6" : "u", # Greek small letter upsilon with perispomeni
    u"\u1FE7" : "u" # Greek small letter upsilon with dialytika and perispomeni
}

# legal punctuation (for lacunae)
legal_punctuation = {
    u"(" : "(",
    u")" : ")", 
    u"." : "."
}


def validate(value, allow):
    invalids = []
    for i, c in enumerate(value):

    # verify character is within the possible general ranges, if not, mark it
    # as invalid and move on otherwise, check to make sure the character is
    # truly valid (ranges are sparsely populated)
    
        cval = ord(c)
        if cval in range(880, 1024) or cval in range(7936, 8192) or c == u' ':
            b = None
            if c == u' ':
                b = c
            if not(b) and ('small' in allow or 'mixed' in allow or 'all' in allow):
                try:
                    b = grek_unaccented_small[c]
                except:
                    pass
            if not(b) and (
                'capital' in allow or 'mixed' in allow or 'all' in allow):
                try:
                    b = grek_unaccented_capital[c]
                except:
                    pass
            if not(b) and ('ancient' in allow or 'all' in allow):
                if 'small' in allow or 'mixed' in allow or 'all' in allow:
                    try:
                        b = grek_accented_ancient_small[c]
                    except:
                        pass
                if not(b) and (
                    'capital' in allow or 'mixed' in allow or 'all' in allow):
                    try:
                        b= grek_accented_ancient_capital[c]
                    except:
                        pass
            if not(b) and ('modern' in allow or 'all' in allow):
                if 'small' in allow or 'mixed' in allow or 'all' in allow:
                    try:
                        b = grek_accented_modern_small[c]
                    except:
                        pass
                if not(b) and (
                    'capital' in allow or 'mixed' in allow or 'all' in allow):
                    try:
                        b = grek_accented_modern_capital[c]
                    except:
                        pass
            if not(b):
                invalids.append({
                    'position': i, 
                    'character': c, 
                    'reason': 'illegal character in appropriate Unicode range'})
        else:
            invalids.append({
                'position': i, 
                'character': c, 
                'reason': 'illegal character outside appropriate Unicode range'})
    return invalids


def transliterate(value):
    transliteration = ''
    for c in value:
        if c == ' ':
            transliteration += c
            continue        
        b = '?'
        try:
            b = grek_unaccented_small[c]
        except:
            try:
                b = grek_unaccented_capital[c]
            except:
                try:
                    b = grek_accented_ancient_small[c]
                except:
                    try:
                        b= grek_accented_ancient_capital[c]
                    except:
                        try:
                            b = grek_accented_modern_small[c]
                        except:
                            try:
                                b = grek_accented_modern_capital[c]
                            except:
                                try:
                                    b = legal_punctuation[c]
                                except:
                                    pass
        transliteration += b
    return transliteration
    
    # what to do about 03D8 - 03E1 which includes various digammas, koppas and 
    # sampis
